#!/bin/bash

# This script will compile the mRF experiment
# It downloads the Sferes2 framework from the SVN, compiles it, then 
# downloads the latest mRF code from any URL, compiles it and runs it.
# This way we can build other sources than the one presents in the SVN:
# no need to commit the mRF experiment each time.

# Download Sferes2 from SVN
svn co https://hotline.isir.upmc.fr/svn/sferes2
svn co https://hotline.isir.upmc.fr/svn/er/src/sferes
cd sferes2/trunk/
rm -rf exp
ln -s ../../sferes/exp 
cp -R ../../sferes/modules .

# Compile sferes
./waf configure
./waf build
./waf check

# Delete existing mRF files
cd exp/mrf
if [ -f mrf.cpp ] # make sure we are in the right folder
  then
    rm wscript
	rm -rf *.pbm
	rm -rf *.cpp
	rm -rf *.hpp
	rm -rf data_analysis
	rm -rf doc
	rm -rf sferes_modif_files
fi

# Download mRF (replace with your own URL if you modified the mRF source files):
wget http://files.wiki4games.com/isir/src.zip
unzip src.zip
cp exp/* .
cd  ../..

# Apply modifications to Sferes2 framework
cp exp/mrf/sferes/gen/evo_float.hpp sferes/gen/evo_float.hpp
cp exp/mrf/modules/nn2/gen_dnn.hpp modules/nn2/gen_dnn.hpp
cp exp/mrf/modules/evoneuro2/lpds.hpp modules/evoneuro2/lpds.hpp
cp exp/mrf/modules/nn2/neuron.hpp modules/nn2/neuron.hpp
cp exp/mrf/modules/nn2/nn.hpp modules/nn2/nn.hpp
cp exp/mrf/sferes/phen/parameters.hpp sferes/phen/parameters.hpp
cp exp/mrf/sferes/misc/sys.hpp sferes/misc/sys.hpp
cp exp/mrf/modules/fastsim/display.cpp modules/fastsim/display.cpp
cp exp/mrf/modules/fastsim/display.hpp modules/fastsim/display.hpp
cp exp/mrf/modules/fastsim/map.cpp modules/fastsim/map.cpp
cp exp/mrf/modules/fastsim/map.hpp modules/fastsim/map.hpp

# Copy a script for qsub in the root folder of the experiment
cp exp/mrf/cluster/run_qsub_mrf.py ../..

# Compile  mRF
./waf --exp mrf

# Run mRF
./build/default/exp/mrf/mrf
