'''
This script is part of the mRF experiment.
Once the mRF program has been compiled on the cluster, you can call this
python script to automatically generate the qsub script and call qsub.

USAGE:
python run_qsub_mrf.py EXPERIMENT_NAME DURATION
* EXPERIMENT_NAME is the name of the experiment (usually better to use the
                  one corresponding to the build options - see mrf.cpp). 
                  Eg 'exp02'.
* DURATION is the maximum duration of the experiment in hours. If the
           the program's running time exceeds this duration, then qsub will
           kill it.
            
EXAMPLE:
* python run_qsub_mrf.py exp02 12

'''


#!/usr/bin/python
# -*- coding: latin-1 -*-

import os
import sys 


def main():
    '''
    This is the main function
    '''
    
    # Parse arguments
    if (len(sys.argv) < 3 ):
        print 'ERROR: you must specify 2 arguments'
        return
    exp_name = str(sys.argv[1]) # Eg. "exp02"
    duration= int(sys.argv[2]) # in hours. Eg. "12"
    
    # Get current folder name
    pwd = os.getcwd()
    pwd_temp = pwd.split('/')
    folder_name = pwd_temp[len(pwd_temp) - 1]
    print folder_name
    
    # Create qsub script
    filename = 'qsub_script_mrf_' + exp_name + '_' + str(duration)
    file = open(filename, 'w')    
    file.write('#!/bin/sh\n')
    file.write('#PBS -N mRF' + exp_name + '_' + str(duration) + '\n')
    file.write('#PBS -o mRF' + exp_name + '_' + str(duration) + '.out\n')
    file.write('#PBS -b mRF' + exp_name + '_' + str(duration) + '.err\n')
    file.write('#PBS -m abe\n')
    file.write('#PBS -M franck.dernoncourt@gmail.com\n')
    file.write('#PBS -l walltime=' + str(duration) + ':00:00\n')
    file.write('#PBS -l ncpus=1\n')
    file.write('./' + folder_name + '/sferes2/trunk/build/debug/exp/mrf/mrf\n')
    file.close()
    
    # Launch qsub script
    os.system('qsub ' + filename)


if __name__ == "__main__":
    main()
